// iolib
// =====
import * as types from '../common/types';
var enterkey = 13;
export function elemOutput(elem) {
    return function (value, newline) {
        setTimeout(function () {
            var result = elem.val() + value;
            if (newline) {
                result += '\n';
            }
            elem.val(result);
        }, 0);
    };
}
export function stdout(value, options, newline) {
    if (newline === void 0) { newline = true; }
    if (options.output === undefined) {
        console.log(value);
        return;
    }
    options.output(value, newline);
}
export function stderr(ex, options) {
    var _a, _b;
    if ((ex.type === types.ti && !options.includeErrors) ||
        (ex.type === types.lib && !options.includeLibErrors)) {
        console.log(ex);
        return;
    }
    var value = '';
    if (ex.type === types.ti) {
        value += 'ERR:';
    }
    else if (ex.type === types.lib) {
        value += 'Error: ';
    }
    else {
        value += 'Unexpected: ';
    }
    value += ex.code;
    if (options.includeLineNumbers && ((_a = ex.source) === null || _a === void 0 ? void 0 : _a.index) !== undefined) {
        value += " on line ".concat(ex.source.index);
    }
    if (options.includeSource && ((_b = ex.source) === null || _b === void 0 ? void 0 : _b.line) !== undefined) {
        value += " :".concat(ex.source.line);
    }
    stdout(value, options);
}
export function onStdin(callback, options) {
    if (options.stdin !== undefined && options.stdin !== '' && options.stdinQueue === undefined) {
        options.stdinQueue = options.stdin.split('\n').reverse();
    }
    var queue = options.stdinQueue;
    if (queue !== undefined) {
        setTimeout(function () {
            var result = queue.pop();
            if (callback(result) === true) {
                onStdin(callback, options);
            }
        }, 0);
        return;
    }
    var input = options.input;
    if (input === undefined) {
        setTimeout(function () {
            if (callback(prompt('Input?')) === true) {
                onStdin(callback, options);
            }
        }, 100);
        return;
    }
    setTimeout(function () { return input.val(''); }, 0);
    input.on('keypress', function (e) {
        var _a;
        if (e.keyCode === enterkey) {
            var result = (_a = input.val()) === null || _a === void 0 ? void 0 : _a.toString();
            setTimeout(function () { return input.val(''); }, 0);
            if (callback(result) !== true) {
                input.off('keypress');
            }
        }
    });
}
export function cleanup(options) {
    if (options.input !== undefined) {
        options.input.off('keypress');
    }
}
