// operation
// =========
import * as core from '../common/core';
import * as types from '../common/types';
export function isTruthy(value) {
    if (value.type === types.NUMBER) {
        return value.float !== 0;
    }
    throw core.DataTypeError;
}
export function deleteVariable(mem, variable) {
    mem.vars.delete(variable.name);
}
export function hasVariable(mem, variable) {
    return mem.vars.has(variable.name);
}
export function assignAns(mem, value) {
    mem.ans = value;
}
export function binaryOperation(left, operator, right) {
    return {
        type: types.BINARY,
        operator: operator,
        left: left,
        right: right,
    };
}
export function resolveNumber(value) {
    var str = '';
    if (value.integer !== undefined) {
        str += value.integer;
    }
    if (value.fraction !== undefined && value.fraction !== null) {
        str += '.' + value.fraction;
    }
    if (value.exponent !== undefined && value.exponent !== null) {
        str += 'e' + value.exponent;
    }
    if (str.length === 0) {
        throw core.libError('Unable to resolve number, no data');
    }
    return parseFloat(str);
}
export function variableToString(variable) {
    var str = variable.name;
    switch (variable.type) {
        case types.VARIABLE:
        case types.STRINGVARIABLE:
            break;
        case types.LISTVARIABLE:
            str = str.substring(types.ListVariablePrefix.length);
            if (!variable.custom) {
                str = "&L".concat(str);
            }
            break;
        default:
            throw core.libError('unexpected variable tostring');
    }
    return str;
}
export function valueToString(value, strict) {
    if (strict === void 0) { strict = false; }
    if (strict && value.type !== types.STRING) {
        throw core.DataTypeError;
    }
    var str = '';
    switch (value.type) {
        case types.NUMBER:
            str = value.float.toString();
            if (str.startsWith('0.')) {
                str = str.substring(1);
            }
            break;
        case types.STRING:
            str = value.chars;
            break;
        case types.LIST:
            str = '{' + value.elements.map(function (e) { return valueToString(e); }).join(' ') + '}';
            break;
        default:
            throw core.libError('unexpected value tostring');
    }
    return str;
}
export function parseDigit(str) {
    if (str === undefined || str === null || str === '' || str.length > 1) {
        return;
    }
    var digit = str.charCodeAt(0) - '0'.charCodeAt(0);
    if (digit < 0 || digit > 9) {
        return;
    }
    return digit;
}
