// looper.node
// ===========
import { Worker } from 'worker_threads';
var worker;
function createWorker() {
    worker = new Worker(new URL('./worker.ts', import.meta.url));
}
function destroyWorker() {
    if (worker !== undefined) {
        worker.terminate();
        worker = undefined;
    }
}
function workerHasListeners() {
    return worker !== undefined && worker.listenerCount('message') > 0;
}
export function post(message) {
    if (worker === undefined) {
        return;
    }
    worker.postMessage(message);
}
export function on(message, listener) {
    if (worker === undefined) {
        createWorker();
    }
    worker.on('message', wrapListener(message, listener));
}
export function off(message, listener) {
    if (worker === undefined) {
        return;
    }
    worker.off('message', wrapListener(message, listener));
    if (!workerHasListeners()) {
        destroyWorker();
    }
}
var listenerMap = new Map();
function wrapListener(message, listener) {
    var secondary = listenerMap.get(message);
    if (secondary === undefined) {
        secondary = new Map();
        listenerMap.set(message, secondary);
    }
    var wrapped = secondary.get(listener);
    if (wrapped === undefined) {
        wrapped = function (postedMessage) {
            if (postedMessage === message) {
                listener();
            }
        };
        secondary.set(listener, wrapped);
    }
    return wrapped;
}
